{# def
name,
id,
multiple=False,
class_="""
bg-neutral-50
block
border
border-neutral-300
p-2.5
rounded-lg
text-base
text-neutral-900
w-full
focus:border-primary-500
focus:ring-primary-500
dark:bg-neutral-700
dark:border-neutral-600
dark:focus:border-primary-500
dark:focus:ring-primary-500
dark:placeholder-neutral-400
dark:text-white
"""
#}

<select name="{{name}}" id="{{id}}" class="{{attrs.class or class_}}" {% if multiple %}multiple{% endif %}>
  {{- content -}}
</select>
