{# def
label,
name,
id,
value,
checked=false,
disabled=false,
onclick="",
class_="""
bg-neutral-100
border-neutral-300
w-4
h-4
text-primary-600
focus:ring-2
focus:ring-primary-500
dark:bg-neutral-700
dark:border-neutral-600
dark:focus:ring-primary-600
dark:ring-offset-neutral-800
""",
label_class="ms-2 text-sm font-medium text-neutral-900 dark:text-neutral-300",
div_class="flex items-center mb-4"
#}
<div class="{{div_class}}">
  <input type="radio" name="{{name}}" id="{{id}}" value="{{value}}" {% if checked %}checked{% endif %}
    class="{{attrs.class or class_}}" {% if onclick %}onclick="{{onclick}}" {% endif %} {%if
    disabled%}disabled{%endif%}>
  <Label for={id} class={label_class}>{{label}}</Label>
</div>
