{# def
type="submit",
id="",
name="action",
value="submit",
hidden=false,
aria_label="",
onclick="",
hx_target="",
hx_swap="",
hx_get="",
hx_select="",
hx_after_request="",
hx_vals="",
full_width=false,
class_="""
bg-primary-600
font-medium
px-5
py-2.5
rounded-lg
text-center
text-sm
text-white
hover:bg-primary-700
focus:outline-none
focus:ring-4
focus:ring-primary-300
dark:bg-primary-600
dark:focus:ring-primary-800
dark:hover:bg-primary-700
"""
#}
<button type="{{type}}" {%if id %}id="{{id}}" {%endif%} name="{{name}}" value="{{value}}" {% if
  hx_target%}hx-target="{{hx_target}}" {% endif %} {% if hx_swap %}hx-swap="{{hx_swap}}" {% endif %} {% if hx_select
  %}hx-select="{{hx_select}}" {% endif %} {% if hx_get %}hx-get="{{hx_get}}" {% endif %} {% if onclick %}onclick="{{onclick}}" {%
  endif %}{% if hx_after_request %}hx-on::after-request="{{hx_after_request}}" {% endif %} {% if hx_vals
  %}hx-vals='{{hx_vals|safe}}' {% endif %} {% if aria_label %}aria-label="{{aria_label}}" {% endif %}
  class="{% if full_width %}w-full {% endif %}{{attrs.class or class_}}" {% if hidden %}hidden{% endif %}>
  {{- content -}}
</button>
